import ij.IJ;
import ij.ImagePlus;
import ij.WindowManager;
import ij.gui.GenericDialog;
import ij.plugin.PlugIn;
import ij.process.ImageProcessor;
import imagescience.images.ColorImage;
import imagescience.images.Dimensions;
import imagescience.images.Image;
import imagescience.transforms.Rotate;
import imagescience.utilities.I5DResource;
import imagescience.utilities.Messenger;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.Panel;
import java.awt.image.ColorModel;

public class TJ_Rotate implements PlugIn {

    private static String zangle = "0.0";
    private static String yangle = "0.0";
    private static String xangle = "0.0";

    private static final String[] schemes = {
	"nearest neighbor",
	"linear",
	"cubic convolution",
	"cubic B-spline",
	"cubic O-MOMS",
	"quintic B-spline"
    };
    private static int scheme = 1;

    private static String bgvalue = "0.0";

    private static boolean adjust = true;
    private static boolean antialias = false;

    public void run(String arg) {

	msg.debug(TJ_Options.debug);
	msg.debug("TransformJ "+TJ_Options.VERSION+": Rotate");

	if (IJ.versionLessThan("1.35i")) return;

	final ImagePlus imp = WindowManager.getCurrentImage();
	if (imp == null)  { error("There are no images open"); return; }

	GenericDialog gd = new GenericDialog("TransformJ: Rotate");
	gd.addStringField("z-angle in degrees:",zangle);
	gd.addStringField("y-angle in degrees:",yangle);
	gd.addStringField("x-angle in degrees:",xangle);
	gd.addPanel(new Panel(),GridBagConstraints.WEST,new Insets(0,0,0,0));
	gd.addChoice("Interpolation scheme:",schemes,schemes[scheme]);
	gd.addStringField("Background value:",bgvalue);
	gd.addCheckbox(" Adjust size to fit result",adjust);
	gd.addCheckbox(" Anti-alias borders",antialias);
	gd.showDialog();

	if (gd.wasCanceled()) return;

	zangle = gd.getNextString();
	yangle = gd.getNextString();
	xangle = gd.getNextString();
	scheme = gd.getNextChoiceIndex();
	bgvalue = gd.getNextString();
	adjust = gd.getNextBoolean();
	antialias = gd.getNextBoolean();

	try {
	    final Image img = Image.wrap(imp);
	    final Rotate rotator = new Rotate();
	    rotator.debug(TJ_Options.debug);
	    double za, ya, xa, bg;
	    try { za = Double.parseDouble(zangle); }
	    catch (Exception e) { throw new IllegalArgumentException("Invalid z-angle for rotation"); }
	    try { ya = Double.parseDouble(yangle); }
	    catch (Exception e) { throw new IllegalArgumentException("Invalid y-angle for rotation"); }
	    try { xa = Double.parseDouble(xangle); }
	    catch (Exception e) { throw new IllegalArgumentException("Invalid x-angle for rotation"); }
	    try { bg = Double.parseDouble(bgvalue); }
	    catch (Exception e) { throw new IllegalArgumentException("Invalid background value"); }
	    rotator.background(bg);
	    int ischeme = Rotate.NEAREST;
	    switch (scheme) {
	    case 0: ischeme = Rotate.NEAREST; break;
	    case 1: ischeme = Rotate.LINEAR; break;
	    case 2: ischeme = Rotate.CUBIC; break;
	    case 3: ischeme = Rotate.BSPLINE3; break;
	    case 4: ischeme = Rotate.OMOMS3; break;
	    case 5: ischeme = Rotate.BSPLINE5; break;
	    }
	    final Image newimg = rotator.run(img,za,ya,xa,ischeme,adjust,antialias);
	    ImagePlus newimp = newimg.imageplus();
	    newimp.setCalibration(imp.getCalibration().copy());
	    final ImageProcessor ip = imp.getProcessor();
	    final ImageProcessor newip = newimp.getProcessor();
	    final ColorModel cm = ip.getColorModel();
	    newip.setColorModel(cm);
	    newimp.getStack().setColorModel(cm);
	    newip.setMinAndMax(ip.getMin(),ip.getMax());
	    boolean i5dinstalled = false;
	    try { Class.forName("Image5D"); i5dinstalled = true; } catch (Exception e) { }
	    if (i5dinstalled && !(newimg instanceof ColorImage)) {
		final Dimensions newdims = newimg.dimensions();
		if (newdims.t > 1 || newdims.s > 1) {
		    newimp = I5DResource.convert(newimp,true);
		    if (I5DResource.instance(imp))
			I5DResource.transfer(imp,newimp);
		}
	    }
	    newimp.changes = TJ_Options.asksave;
	    newimp.show();

	} catch (OutOfMemoryError e) {
	    error("Not enough memory for this operation");

	} catch (IllegalArgumentException e) {
	    error(e.getMessage());
	}
    }

    private void error(final String message) { msg.error("TransformJ: Error", message+"."); }

    private final Messenger msg = new Messenger();

}
